#
#     File: Functions_LC.r
#
#     =====================================================
#
#     Function to update fit in a GLM with constraints and smoothing
#
#     NB. Poisson errors assumed
#
#     X.   = regression matrix
#     V.   = vector, data present/missing
#     OFF. = offset
#     D.   = vector of observed deaths
#     A.   = a matrix, ncol = 1, current estimate of regression
#            coefficients
#     H.   = Constraint vector
#     C.   = vector of constraint values, length = nrow(H.)
#     L.   = Current estimate of Lagrange multiplier
#     P.   = penalty matrix
#
Update.Con.Pen <- function(X., V., OFF., D., A., H., C., L., P.){
  Ita <- c(OFF. + X. %*% A.)
  Mu <- exp(Ita)
  Wt <- Mu * V.
  Bt.W.B <- t(X.) %*% (Wt * X.)
  Lower.Right <- matrix(0, nrow = nrow(H.), ncol = nrow(H.))
  Row.1 <- cbind(Bt.W.B + P.,  t(H.))
  Row.2 <- cbind(H., Lower.Right)
  Lhs <- rbind(Row.1, Row.2)
  Rhs <- c(Bt.W.B %*% A. + t(X.) %*% (D. - Mu), C.)
  a <- solve(Lhs, Rhs)
  matrix(a, ncol = 1)
}
#
#     ============================================================
#
#     Function to estimate parameters in LC models
#
#     Arguments
#
#     X.MAT.    : List of regression matrices for alpha, beta and kappa
#     V.        : Weight vector, can be used to set missing values
#     OFF.      : log(exposure)
#     D.        : Deaths  
#     A.INIT.   : List of Initial values for a, b and kappa
#     H.        : List of constraint matrices acting on b and kappa 
#     C.        : List of constraint constants for beta and kappa
#     L.Init    : List of initial values for Lagrange multipliers
#                 for beta and kappa
#     P.        : List of penalty matrices on a and b
#     TOL.      : Convergence criterion
#     MONITOR:  : Output estimated coefficients in iteration, TRUE/FALSE
#     MAX.ITER. : Maximum number of iterations
#
Estimate.LC <- function(X.MAT., V., OFF., D., A.INIT., H., C., L.INIT.,
                        P., TOL., MONITOR., MAX.ITER.){
  X.a <- X.MAT.[[1]]
  X.b <- X.MAT.[[2]]
  X.k <- X.MAT.[[3]]
  n.a <- nrow(X.a)
  n.y <- nrow(X.k)
  P1 <- dbind(list(P.[[1]], 0*X.k))   #  Penalty matrix on a and kappa
  P2 <- P.[[2]]                       #  penalty matrix on b
  c.a.1 <- ncol(X.a)
  A <- A.INIT.[[1]]
  Alpha <- X.a %*% A
  c.a.2 <- ncol(X.b)
  B <- A.INIT.[[2]]
  Beta <- X.b %*% B
  H.B <- matrix(c(H.[[2]]), nrow = 1)
  C.Beta <- C.[[1]]
  L.Beta <- L.INIT.[[1]]
  Kappa <- A.INIT.[[3]]
  H.A.Kappa <-  matrix(c(H.[[1]], H.[[3]]), nrow = 1)
  H.B <- matrix(c(H.[[2]]), nrow = 1)
  C.Kappa <- C.[[2]]
  L.Kappa <- L.INIT.[[2]]
  One.Y <- rep(1, n.y)
  Iter <- 0
  Tol <- 1
  if (MONITOR. == TRUE) cat("Iter     a[1]            tol", "\n")
  while(Tol > TOL. && Iter < MAX.ITER.){
    Iter <- Iter + 1
    #   Update Alpha & Kappa
    temp <- c(Update.Con.Pen(cbind(kronecker(One.Y, X.a),
                             kronecker(X.k, Beta)), V., OFF., D.,
                             c(A, Kappa), H.A.Kappa, C.Kappa, L.Kappa, P1))
    A.New <- temp[1:c.a.1]
    Alpha.New <- X.a %*% A.New
    Kappa.New <- temp[(c.a.1+1):(c.a.1+n.y)]
    L.Kappa <- temp[c.a.1+n.y+1]
    #     Update Beta
    temp <- c(Update.Con.Pen(kronecker(Kappa.New, diag(n.a)) %*% X.b,
                             V., OFF. + c(kronecker(One.Y, Alpha.New)), D.,
                             B, H.B, C.Beta, L.Beta, P2))
    B.New <- temp[1:c.a.2]
    Beta.New <- X.b %*% B.New
    L.Beta <- temp[c.a.2+1]
    Theta <- c(Alpha, Beta, Kappa)
    Theta.New <- c(Alpha.New, Beta.New, Kappa.New)
    Tol <- max(abs(Theta - Theta.New))
    A <- A.New
    Alpha <- Alpha.New
    B <- B.New
    Beta <- Beta.New
    Kappa <- Kappa.New
    if (MONITOR. == TRUE) cat(Iter, Alpha[1], Tol, "\n")
  }
  if (Iter == MAX.ITER.) warning("Failed to converge")
  return(list(Theta = matrix(Theta.New, ncol = 1)))
}
#
#     Function to calculate output in LC model
#
#     Arguments as Estimate.LC and
#
#     LAMBDA. : Values of smoothing parameters
#     TRACE.  : TRUE, print current values of lambda and BIC, else FALSE
#
Calc.LC <- function(X.MAT., V., OFF., D., A.INIT., H., C., L.INIT., P.,
                    LAMBDA., TOL., MONITOR., MAX.ITER., TRACE.){
  P1 <- LAMBDA.[1] * P.[[1]]  # Penalty on a
  P2 <- LAMBDA.[2] * P.[[2]]  # Penalty on b
  P <- list(P1, P2)  
  temp <- Estimate.LC(X.MAT., V., OFF., D., A.INIT., H., C., L.INIT., P,
                      TOL., MONITOR., MAX.ITER.)
  n.a <- nrow(X.MAT.[[1]])
  n.y <- nrow(X.MAT.[[3]])
  Theta <- temp$Theta
  Alpha <- temp$Theta[1:n.a]
  Beta <- temp$Theta[(n.a+1) : (2*n.a)]
  Kappa <- temp$Theta[(2*n.a+1) : (2*n.a+n.y)]
  Ita <- matrix(Alpha,ncol = 1) %*% matrix(1,nrow=1,ncol=n.y) +
                     matrix(Beta,ncol = 1) %*% matrix(Kappa, nrow = 1)
  D.Hat <- exp(OFF. + c(Ita))
  Dev <- 2* sum(D. * log(D./D.Hat) - (D. - D.Hat))
  #
  #    Calculate trace of smooth part for beta
  #  
  X.Mat <- kronecker(Kappa, diag(n.a)) %*% X.MAT.[[2]]
  H.B <- matrix(H.[[2]], nrow = 1)
  Delta <- t(X.Mat) %*% (D.Hat * X.Mat) + P2 + t(H.B) %*% H.B
  Delta.Inv <- solve(Delta)
  Psi <- Delta.Inv - Delta.Inv %*% t(H.B) %*%
    solve(H.B %*% Delta.Inv %*% t(H.B)) %*% H.B %*% Delta.Inv
  Tr2 <- ncol(X.Mat) - 1 - sum(diag(Psi %*% P2))
  #
  #    Calculate trace of smooth part of alpha and for kappa
  #
  P1 <- dbind(list(P1, 0 * diag(n.y))) # penalty on a and kappa
  One.Y <- rep(1, n.y)
  X.Mat <- cbind(kronecker(One.Y, X.MAT.[[1]]), kronecker(diag(n.y), Beta))
  H.Kappa <- matrix(c(H.[[1]], H.[[3]]), nrow = 1)
  Delta <- t(X.Mat) %*% (D.Hat * X.Mat) + P1 + t(H.Kappa) %*% H.Kappa
  Delta.Inv <- solve(Delta)
  Psi <- Delta.Inv - Delta.Inv %*% t(H.Kappa) %*%
    solve(H.Kappa %*% Delta.Inv %*% t(H.Kappa)) %*% H.Kappa %*% Delta.Inv
  Tr1 <- ncol(X.Mat) - 1 - sum(diag(Psi %*% P1))
  Tr <- Tr1 + Tr2
  Bic <- Dev + log(sum(V.)) * Tr
  if(TRACE. == TRUE) cat(LAMBDA., Bic, "\n")
  return(list(Alpha = Alpha, Beta = Beta, Kappa = Kappa, Ita = Ita,
              Dev = Dev, Tr = Tr, Bic = Bic))
}




