#
#   File: LC_Models.r
#
#   Fits LC models in the original, the DDE and the LC(S) versions
#
#   Deviances in Table 1 for Poisson model are obtained and
#   Figures 1, 2, 3 and 4 are produced.
#
#   ===================================================
#
#     Run setup file: load data, functions, calculate the lists of
#     regression, penalty and constraint matrices.
#     See the file for more detail.
#
source("LC_Setup.r")
#
#     ====================================================================
#
#     Fit LC model with Calc.LC
#
cat("\n", "Fit LC original", "\n", "\n")
Ans.LC <- Calc.LC(X.LC, V, Off, Dth.V, Coef.Init.LC, H.LC, C.List, L.Init,
                  P.LC, c(0,0), 10^(-12), TRUE, 50, TRUE)
names(Ans.LC)
cat("Output deviance for LC model", Ans.LC$Dev, "\n")
cat("This is Dev_P from Table 1", "\n")
#
#   Save coefficients and fitted log(mortality) table
#
Alpha.LC <- Ans.LC$Alpha
Beta.LC <- Ans.LC$Beta
Kappa.LC <- Ans.LC$Kappa
Fit.Log.M.LC <- Ans.LC$Ita
#
# Optionally save results
#
# save(Alpha.LC, Beta.LC, Kappa.LC, Fit.Log.M.LC, file = "Results.LC.RData")
#
#    Plot Figure 1 with above initial estimates rather than AP estimates
#
par(ask = TRUE)
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
plot(Age, Alpha.Init, ylab = "Alpha", cex = 0.8, col = "red")
lines(Age, Alpha.LC, col = "blue")
legend("topleft", legend = c("Initial estimates", "LC estimates"), 
       pch = c(1,-1), lty = c(-1,1), col = c("red", "blue"))
#
plot(Year, Kappa.Init, ylab = "Kappa", cex = 0.8, col = "red")
lines(Year, Kappa.LC, col = "blue")
#
plot(Age, Beta.LC, type = "l", col = "blue", ylab = "Beta")
points(Age, Beta.Init, cex = 0.8, col = "red")
#  
AGE <- 65  # Set plotting age
Row <- AGE - min(Age) + 1
plot(Year, Fit.Log.M.LC[Row, ], xlab = "Year", ylab = "log(mortality)", 
     type = "l", col = "blue")
points(Year, Obs[Row, ], cex = 0.8, col = "green")
legend("topright", legend = c("Data", "LC"), pch = c(1, -1), lty = c(-1, 1),
       col = c("green", "blue"))
legend("bottomleft", legend = paste("Age", AGE), bty = "n")
#
#     End original LC
#
#     ================================================================
#
#     Fit DDE version of LC model, ie, smooth beta
#
#     Function to calculate BIC for given lambda
#
cat("\n", "Obtain smoothing parameter in DDE model", "\n", "\n")
Calc.BIC <- function(X){
  Calc.LC(X.DDE, V, Off, Dth.V, Coef.Init.DDE, H.DDE, C.List, L.Init, P.DDE,
          c(0, X), 10^(-4), FALSE, 20, TRUE)$Bic
}
#
#    Plot BIC values over a grid from 10^5 to 10^7
#
par(mfrow = c(1,1))
BIC <- NULL
SEQ <- seq(5, 7, by = 0.1)
for (i in 1:length(SEQ)){
  BIC <- c(BIC, Calc.BIC(10^SEQ[i]))
}
plot(SEQ, BIC, xlab = "log(lambda(b))")
#
#    Optimal value of lambda = 10^6
#
#    Fit DDE model with Calc.LC
#
cat("\n", "Fit DDE", "\n", "\n")
Ans.DDE <- Calc.LC(X.DDE, V, Off, Dth.V, Coef.Init.DDE, H.DDE, C.List, L.Init,
                   P.DDE, c(0, 10^6), 10^(-12), TRUE, 50, TRUE)
names(Ans.DDE)
cat("Output deviance for DDE model", Ans.DDE$Dev, "\n")
cat("This is Dev_P from Table 1", "\n")
#
#   Save coefficients and fitted log(mortality) table
#
Alpha.DDE <- Ans.DDE$Alpha
Beta.DDE <- Ans.DDE$Beta
Kappa.DDE <- Ans.DDE$Kappa
Fit.Log.M.DDE <- Ans.DDE$Ita
#
# Optionally save results
#
# save(Alpha.DDE, Beta.DDE, Kappa.DDE, Fit.Log.M.DDE, file = "Results.DDE.RData")
#
#    Plot Figure 2 left
#
plot(Age, Beta.LC, col = "red", ylab = "Beta")
lines(Age, Beta.DDE, col = "blue", lwd = 2)
legend("topright", legend = c("LC", "DDE"), pch = c(1, -1), lty = c(-1, 1),
       lwd = c(-1, 2), col = c("red", "blue"))
#
#     ==================================================
#
#     Fit LC(S) version of LC model, ie, smooth alpha and beta
#
#     We find optimal values of smoothing parameters by a simple grid search
#     first over a and then over b
#
cat("\n", "Obtain smoothing parameters in LC(S) model", "\n", "\n")
Calc.BIC.A <- function(X){
  Calc.LC(X.LC.S, V, Off, Dth.V, Coef.Init.LC.S, H.LC.S, C.List, L.Init,
          P.LC.S, c(X, 10^6), 10^(-4), FALSE, 20, TRUE)$Bic
}
par(mfrow = c(1,1))
BIC <- NULL
SEQ <- seq(120, 125, by = 0.5)
for (i in 1:length(SEQ)){
  BIC <- c(BIC, Calc.BIC.A(SEQ[i]))
}
plot(SEQ, BIC, xlab = "log(lambda(a))")
#
Calc.BIC.B <- function(X){
  Calc.LC(X.LC.S, V, Off, Dth.V, Coef.Init.LC.S, H.LC.S, C.List, L.Init,
          P.LC.S, c(123, X), 10^(-4), FALSE, 20, TRUE)$Bic
}
par(mfrow = c(1,1))
BIC <- NULL
SEQ <- seq(5, 7, by = 0.1)
for (i in 1:length(SEQ)){
  BIC <- c(BIC, Calc.BIC.B(10^SEQ[i]))
}
plot(SEQ, BIC, xlab = "log(lambda(b))")
#
#     Optimal values of smoothing parameter: lambda_a = 123, lambda_b = 10^6

Opt.L <- c(123, 10^6)
cat("\n", "Fit LC(S) model", "\n", "\n")
Ans.LC.S <- Calc.LC(X.LC.S, V, Off, Dth.V, Coef.Init.LC.S, H.LC.S,
                    C.List, L.Init, P.LC.S, Opt.L, 10^(-12), FALSE, 50, TRUE)
names(Ans.LC.S)
cat("Output deviance for LC(S) model", Ans.LC.S$Dev, "\n")
cat("This is Dev_P from Table 1", "\n")
#
#   Save coefficients and fitted log(mortality) table
#
Alpha.LC.S <- Ans.LC.S$Alpha
Beta.LC.S <- Ans.LC.S$Beta
Kappa.LC.S <- Ans.LC.S$Kappa
Fit.Log.M.LC.S <- Ans.LC.S$Ita
#
# Optionally save results
#
# save(Alpha.LC.S, Beta.LC.S, Kappa.LC.S, Fit.Log.M.LC.S,
#      file = "Results.LC.S.RData")
#
#     Forecast Kappa with arima(1,1,1) model to 2050
#
Forecast.Year <- 2050
temp <- arima(Kappa.LC, c(1,1,1), method = "ML")
Kappa.LC.Pred <- predict(temp, Forecast.Year - max(Year))$pred
#
temp <- arima(Kappa.DDE, c(1,1,1), method = "ML")
Kappa.DDE.Pred <- predict(temp, Forecast.Year - max(Year))$pred
#
temp <- arima(Kappa.LC.S, c(1,1,1), method = "ML")
Kappa.LC.S.Pred <- predict(temp,Forecast.Year  - max(Year))$pred
Year.F <- (max(Year)+1):Forecast.Year
#
#   Obtain forecast mortality tables
#
Ita.LC <- matrix(Alpha.LC, ncol = 1) %*%
  matrix(1,nrow=1, ncol=length(Year.F)) +
  matrix(Beta.LC, ncol = 1) %*% matrix(Kappa.LC.Pred, nrow = 1)
#
Ita.DDE <- matrix(Alpha.DDE, ncol = 1) %*%
  matrix(1,nrow=1, ncol=length(Year.F)) +
  matrix(Beta.DDE, ncol = 1) %*% matrix(Kappa.DDE.Pred, nrow = 1)
#
Ita.LC.S <- matrix(Alpha.LC.S,ncol = 1) %*%
  matrix(1,nrow=1, ncol=length(Year.F)) +
  matrix(Beta.LC.S,ncol = 1) %*% matrix(Kappa.LC.S.Pred, nrow = 1)
#
#    Plots
#
#    Fig 2 right, crossover of LC and DDE at ages 41 and 42
#
AGE <- 41
Row <- AGE - min(Age) + 1
plot(Year.F, Ita.LC[Row, ], xlab = "Year", ylab = "log(mortality)",
     type = "l", lwd = 2, ylim = c(-7.2, -6.4), col = "red")
lines(Year.F, Ita.DDE[Row, ], lty = 2, lwd = 2, col = "blue")
#
AGE <- 42
Row <- AGE - min(Age) + 1
lines(Year.F, Ita.LC[Row, ], lwd = 2, col = "red")
lines(Year.F, Ita.DDE[Row, ], lty = 2, lwd = 2, col = "blue")
legend("topright", legend = c("LC", "DDE"), lty = c(1,2), lwd = 2,
       col = c("red", "blue"))
text(2015, -6.7, "Age: 41")
text(2015, -6.45, "Age: 42")
#
#    Fig 3 left
#
AGE <- 50
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
plot(Year.F, Ita.LC[Row, ], xlab = "Year", ylab = "log(mortality)",
     type = "l", lwd = 2, ylim = c(-6.7, -5.4))
AGE <-AGE + 1 ; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.LC[Row, ], lwd = 2, lty = 2)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.LC[Row, ], lwd = 2, lty = 3)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.LC[Row, ], lwd = 2, lty = 4)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.LC[Row, ], lwd = 2, lty = 5)
legend("bottomleft", legend = c("Age = 51", "Age = 52", "Age = 53",
                       "Age = 54", "Age = 55"), lty = 1:5, lwd = 2)
legend("topright", legend = "LC     ", bty = "n", cex = 1.2)
#
#    Fig 3 right
#
AGE <- 50
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
plot(Year.F, Ita.DDE[Row, ], xlab = "Year", ylab = "log(mortality)",
     type = "l", lwd = 2, ylim = c(-6.75, -5.4))
AGE <-AGE + 1 ; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.DDE[Row, ], lwd = 2, lty = 2)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.DDE[Row, ], lwd = 2, lty = 3)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.DDE[Row, ], lwd = 2, lty = 4)
AGE <- AGE + 1; Row <- AGE - min(Age) + 1
lines(Year.F, Ita.DDE[Row, ], lwd = 2, lty = 5)
legend("topright", legend = "DDE    ", bty = "n", cex = 1.2)
legend("bottomleft", legend = c("Age = 51", "Age = 52", "Age = 53",
                       "Age = 54", "Age = 55"), lty = 1:5, lwd = 2)
#
#    Fig 4 left: first forecast year
#
Place <- 1
plot(Age[-1]-0.5, diff(Ita.LC[, Place]),
     xlab = "Age", ylab = "Mortality difference", col = "red")
points(Age[-1]-0.5, diff(Ita.DDE[, Place]), pch = 2, col = "blue")
lines(Age[-1]-0.5, diff(Ita.LC.S[, Place]), col = "green", lwd = 2)
legend("bottomright", legend = c("LC", "DDE", "LC(S)"), lty = c(-1,-1,1),
       pch = c(1,2,-1), col = c("red", "blue", "green"), lwd = c(-1,-1,2))
#
#    Fig 4 right: last forecast year
#
Place <- 41
plot(Age[-1]-0.5, diff(Ita.LC[, Place]),
     xlab = "Age", ylab = "Mortality difference", col = "red")
points(Age[-1]-0.5, diff(Ita.DDE[, Place]), pch = 2, col = "blue")
lines(Age[-1]-0.5, diff(Ita.LC.S[, Place]), col = "green", lwd = 2)
legend("bottomright", legend = c("LC", "DDE", "LC(S)"), lty = c(-1,-1,1),
       pch = c(1,2,-1), col = c("red", "blue", "green"), lwd = c(-1,-1,2))


