#
#   File: LC_Setup.r
#
#   Reads in data, loads function files and spline library.
#   Sets up smoothing, initializes estimates and defines lists of
#
#      (a) regression matrices
#      (b) penalty matrices
#      (c) constraint matrices
#
#   for the three models
#
rm(list = ls())
#
#   Load ONS data
#    
load(file = "ONS_40_90_1961_2009.RData")
ls()
#
#    Load function files
#
source("Functions.r")          # Load some basic functions 
source("Functions_LC.r")       # Functions to fit LC models
#
#   Load spline library
#
library(splines)
#
#    Set spline parameters
#
bdeg <- 3        #  Best left as 3
pord <- 2        #  Best left as 2 
Age.dx <- 5      #  Could be varied
#
#    Calculate B-spline regression matrix, B.a
#
Domain.a <- Domain.Age(min(Age), max(Age), Age.dx)$Domain
Setup.a <- bspline(Age, min(Domain.a), max(Domain.a), Age.dx, bdeg)
B.a <- Setup.a$B     
#
#    Find penalty matrix, DtD
#
D.Mat <- diag(ncol(B.a))
for(k in 1:pord) D.Mat <- diff(D.Mat)
DtD <- t(D.Mat) %*% D.Mat
#
#   Obtain some basic variables
#
Dth.V <- c(Dth)
Exp.V <- c(Exp)
n.a <- length(Age)
n.y <- length(Year)
One.A <- rep(1, n.a)             
One.Y <- rep(1, n.y)
#
#    Initialize by finding row and column means - see Section 3.2
#
Obs <- log(Dth/Exp)
Alpha.Init <- apply(Obs, 1, mean)
Kappa.Init <- apply(Obs, 2, mean)
Kappa.Init <- Kappa.Init - mean(Kappa.Init)  # Centre Kappa
Kappa.Init <- n.a * Kappa.Init               # and scale
Beta.Init <- rep(1, n.a)/n.a                 # Sum = 1
#
#    Calculate offset, Off, and weight, V
#
Off <- log(Exp.V)
V <- rep(1, length(Dth.V))   #  Note: Can be set for missing values
#
#    Define constraint values - these are fixed across models
#
C.Kappa <- 0         #  Sum to zero
C.Beta <- 1          #  Sum to one
C.List  <- list(C.Beta, C.Kappa)
#
#    Initialize Lagrange multipliers - these are fixed across models
# 
L.Kappa.Init <- 0
L.Beta.Init <- 1
L.Init <- list(L.Beta.Init, L.Kappa.Init)
#
#    Define lists (a), (b) and (c) for each model in turn
#
#    Original LC model
#
X.LC <- list(diag(n.a), diag(n.a), diag(n.y))   # Regression
P.LC <- list(diag(n.a), diag(n.a))              # Penalty
H.Alpha <- matrix(rep(0, n.a),  nrow = 1)
H.Beta <- matrix(rep(1, n.a),  nrow = 1)
H.Kappa <- matrix(rep(1, n.y), nrow = 1)
H.LC <- list(H.Alpha, H.Beta, H.Kappa)          # Constraints
Coef.Init.LC <- list(Alpha.Init, Beta.Init, Kappa.Init)   # Initialize
#
#    DDE model
#
X.DDE <- list(diag(n.a), B.a, diag(n.y))        # Regression
P.DDE <- list(diag(n.a), DtD)                   # Penalty
H.B <- matrix(rep(1, n.a),  nrow = 1) %*% B.a   # Constraint on b, not beta
H.DDE <- list(H.Alpha, H.B, H.Kappa)            # Constraints
B.Init <- solve(t(B.a) %*% B.a, t(B.a) %*% Beta.Init)
Coef.Init.DDE <- list(Alpha.Init, B.Init, Kappa.Init)   # Initialize
#
#    LC(S)
#
X.LC.S <- list(B.a, B.a, diag(n.y))             # Regression
P.LC.S <- list(DtD, DtD)                        # Penalty
H.A <- matrix(rep(0, ncol(B.a)),  nrow = 1)     # No constraint on a
H.LC.S <- list(H.A, H.B, H.Kappa)               # Constraints
A.Init <- solve(t(B.a) %*% B.a, t(B.a) %*% Alpha.Init)
Coef.Init.LC.S <- list(A.Init, B.Init, Kappa.Init)   # Initialize
